@@ -33,6 +33,8 @@
 #' @param xlab Label for x axis
 #' @param ylab Label for y axis
 #' @param zlab Label for z axis
+#' @param main the main title of the plot
+#' @param cex.main Factor for main title size
 #' @param surface Method for the calculation of the surface z values. "predict" takes the predicted values from the model, "smooth" uses a thin plate smoother (function \code{Tps} from the \code{fields} package) of the raw data
 #' @param lambda lambda parameter for the smoother. Default (NULL) means that it is estimated by the smoother function. Small lambdas around 1 lead to rugged surfaces, big lambdas to very smooth surfaces.
 #' @param rotation Rotation of the 3d surface plot (when type == "3d")
@@ -40,23 +42,23 @@
 #' @param gridsize Number of grid nodes in each dimension
 #' @param bw Print surface in black and white instead of colors?
 #' @param legend Print color legend for z values?
-#' @param cex Font size factor for axes labels
+#' @param cex Font size factor for axes labels, axes titles, and the main title
 #' @param type \code{3d} for 3d surface plot, \code{contour} for 2d contour plot, "interactive" for interactive rotatable plot. Shortcuts (i.e., first letter of string) are sufficient
 #' @param points A list of parameters which define the appearance of the raw scatter points: 
 #'	\itemize{
-#'		\item{data: Data frame which contains the coordinates of the raw data points. FIrst column = x, second = y, third = z. This data frame is automatically generated when the plot is based on a fitted RSA-object}
-#'		\item{show = TRUE: Should the original data points be overplotted?}
-#' 		\item{value="raw": Plot the original z value, "predicted": plot the predicted z value}
-#'		\item{jitter = 0: Amount of jitter for the raw data points. For z values, a value of 0.005 is reasonable}
-#'		\item{cex = .5: multiplication factor for point size}
-#' 		\item{out.mark = FALSE: If set to TRUE, outliers according to Bollen & Jackman (1980) are printed as red X symbols. This option works regardless of whether the RSA function has set out.rm to TRUE or FALSE: 
+#'		\item data: Data frame which contains the coordinates of the raw data points. First column = x, second = y, third = z. This data frame is automatically generated when the plot is based on a fitted RSA-object
+#'		\item show = TRUE: Should the original data points be overplotted?
+#'		\item color = "black": Color of the points
+#' 		\item value="raw": Plot the original z value, "predicted": plot the predicted z value
+#'		\item jitter = 0: Amount of jitter for the raw data points. For z values, a value of 0.005 is reasonable
+#'		\item cex = .5: multiplication factor for point size
+#' 		\item out.mark = FALSE: If set to TRUE, outliers according to Bollen & Jackman (1980) are printed as red X symbols, but only when they have been removed in the RSA function: \code{RSA(..., out.rm=TRUE)}.
 #'			\itemize{
-#'				\item{If out.rm == TRUE (in RSA()) and out.mark == FALSE (in plotRSA()), the outlier is removed from the model and *not plotted* in plotRSA.}
-#'				\item{If out.rm == TRUE (in RSA()) and out.mark == TRUE (in plotRSA()), the outlier is removed from the model but plotted and marked in plotRSA.}
-#'				\item{If out.rm == FALSE (in RSA()) and out.mark == FALSE (in plotRSA()), the outlier is not removed from the model and plotted as a normal point in plotRSA (but not marked as outlier).}
-#'				\item{If out.rm == FALSE (in RSA()) and out.mark == TRUE (in plotRSA()), the outlier is not removed from the model, but plotted and marked in plotRSA.}
-#'				\item{Example syntax: \code{plotRSA(r1, points=list(show=TRUE, out.mark=TRUE))}}
-#'		}}
+#'				\item If out.rm == TRUE (in RSA()) and out.mark == FALSE (in plotRSA()), the outlier is removed from the model and *not plotted* in plotRSA.
+#'				\item If out.rm == TRUE (in RSA()) and out.mark == TRUE (in plotRSA()), the outlier is removed from the model but plotted and marked in plotRSA.
+#'				\item If out.rm == FALSE (in RSA()): Outliers are not removed and cannot be plotted.
+#'				\item Example syntax: \code{plotRSA(r1, points=list(show=TRUE, out.mark=TRUE))}
+#'		}
 #'	}
 
 #' @param model If x is an RSA object: from which model should the response surface be computed?
@@ -69,14 +71,14 @@
 #' @param link Link function to transform the z axes. Implemented are "identity" (no transformation; default), "probit", and "logit"
 #' @param border Should a thicker border around the surface be plotted? Sometimes this border leaves the surrounding box, which does not look good. In this case the border can be suppressed by setting \code{border=FALSE}.
 #' @param contour A list defining the appearance of contour lines (aka. height lines). show=TRUE: Should the contour lines be plotted on the 3d wireframe plot? (Parameter only relevant for \code{type="3d"}). color = "grey40": Color of the contour lines. highlight = c(): A vector of heights which should be highlighted (i.e., printed in bold). Be careful: the highlighted line is not necessarily exactly at the specified height; instead the nearest height line is selected.
-#' @param hull Plot a bag plot on the surface (This is a bivariate extension of the boxplot. 50% of points are in the inner bag, 50% in the outer region). See Rousseeuw, Ruts, & Tukey (1999).
+#' @param hull Plot a bag plot on the surface (This is a bivariate extension of the boxplot. 50\% of points are in the inner bag, 50\% in the outer region). See Rousseeuw, Ruts, & Tukey (1999).
 #' @param SP.CI Plot the CI of the stationary point (only relevant for \code{type="contour"})
 #' @param distance A vector of three values defining the distance of labels to the axes
 #' @param tck A vector of three values defining the position of labels to the axes (see ?wireframe)
-#' @param pal A palette for shading
+#' @param pal A palette for shading. You can use \code{\link{colorRampPalette}} to construct a color ramp, e.g. \code{plot(r.m, pal=colorRampPalette(c("darkgreen", "yellow", "darkred"))(20))}
 #' @param pal.range Should the color range be scaled to the box (\code{pal.range = "box"}, default), or to the min and max of the surface (\code{pal.range = "surface"})? If set to "box", different surface plots can be compared along their color, as long as the zlim is the same for both.
 #' @param pad Pad controls the margin around the figure (positive numbers: larger margin, negative numbers: smaller margin)
-#'#' @param ... Additional parameters passed to the plotting function (e.g., sub="Title"). A useful title might be the R squared of the plotted model: sub = as.expression(bquote(R^2==.(round(getPar(x, "r2", model="full"), 3))))
+#'#' @param ... Additional parameters passed to the plotting function (e.g., sub="Title"). A useful title might be the R squared of the plotted model: \code{sub = as.expression(bquote(R^2==.(round(getPar(x, "r2", model="full"), 3))))}
 #'
 #' @references
 #' Rousseeuw, P. J., Ruts, I., & Tukey, J. W. (1999). The Bagplot: A Bivariate Boxplot. The American Statistician, 53(4), 382-387. doi:10.1080/00031305.1999.10474494
@@ -102,17 +104,17 @@
 #' df <- within(df, {
 #' 	diff <- x-y
 #' 	absdiff <- abs(x-y)
-#' 	sqdiff <- (x-y)^2
+#' 	SD <- (x-y)^2
 #' 	z.diff <- diff + rnorm(n, 0, err)
 #' 	z.abs <- absdiff + rnorm(n, 0, err)
-#' 	z.sq <- sqdiff + rnorm(n, 0, err)
+#' 	z.sq <- SD + rnorm(n, 0, err)
 #' 	z.add <- diff + 0.4*x + rnorm(n, 0, err)
 #' 	z.complex <- 0.4*x + - 0.2*x*y + + 0.1*x^2 - 0.03*y^2 + rnorm(n, 0, err)
 #' })
 #' 
-#' r1 <- RSA(z.sq~x*y, df, models=c("sqdiff", "full", "IA"))
+#' r1 <- RSA(z.sq~x*y, df, models=c("SQD", "full", "IA"))
 #' plot(r1)	# default: model = "full"
-#' plot(r1, model="sqdiff", points=list(show=TRUE, value="predicted"))
+#' plot(r1, model="SQD", points=list(show=TRUE, value="predicted"))
 
 
 
@@ -125,17 +127,18 @@
 # distance=c(1, 1, 1), tck=c(1, 1, 1)
 
 plotRSA <- function(x=0, y=0, x2=0, y2=0, xy=0, w=0, wx=0, wy=0, x3=0, xy2=0, x2y=0, y3=0, b0=0, type="3d", model="full", 
-	xlim=NULL, ylim=NULL, zlim=NULL, xlab=NULL, ylab=NULL, zlab=NULL, 
+	xlim=NULL, ylim=NULL, zlim=NULL, 
+	xlab=NULL, ylab=NULL, zlab=NULL, main="",
 	surface="predict", lambda=NULL, 
 	rotation=list(x=-63, y=32, z=15), label.rotation=list(x=19, y=-40, z=92), 
 	gridsize=21, bw=FALSE, legend=TRUE, param=TRUE, 
 	axes=c("LOC", "LOIC", "PA1", "PA2"), project=FALSE, maxlines=FALSE,
-	cex=1.2,
+	cex=1, cex.main=1, 
 	points = list(data=NULL, show=NA, value="raw", jitter=0, color="black", cex=.5, out.mark=FALSE),
 	demo=FALSE, fit=NULL, link="identity", 
 	tck=c(1.5, 1.5, 1.5), distance=c(1.3, 1.3, 1.4), border=TRUE, 
 	contour = list(show=FALSE, color="grey40", highlight = c()),
-	hull=FALSE, SP.CI=FALSE, 
+	hull=NA, SP.CI=FALSE, 
 	pal=NULL, pal.range="box", 
 	pad=0, ...) {
 	
@@ -159,11 +162,26 @@ plotRSA <- function(x=0, y=0, x2=0, y2=0, xy=0, w=0, wx=0, wy=0, x3=0, xy2=0, x2
 		warning("You must provide a data frame with the coordinates of the raw data points (points = list(show = TRUE, data = ???)). Points are not plotted.")
 		points$show <- FALSE
 	}
+	if (!is.null(fit)) {
+		if (points$out.mark==TRUE & fit$out.rm==FALSE) {
+			warning("Outliers can only be marked in the plot when they were removed in the RSA function: RSA(..., out.rm=TRUE).")
+			points$out.mark <- FALSE
+		}
+	}
 		
 	if (is.null(contour$show)) contour$show <- TRUE
 	if (is.null(contour$color)) contour$color <- "grey40"
 	if (is.null(contour$highlight)) contour$highlight <- c()
 	
+	# define default behavior for the "hull" parameter
+	if (is.na(hull)) {
+		if (!is.null(fit) | !is.null(points$data)) {
+			hull <- TRUE
+		} else {
+			hull <- FALSE
+		}
+	}
+	
 	type <- match.arg(type, c("interactive", "3d", "contour"))
 	surface <- match.arg(surface, c("predict", "smooth"))
 	points[["value"]] <- match.arg(points[["value"]], c("raw", "predicted"))
@@ -201,7 +219,7 @@ plotRSA <- function(x=0, y=0, x2=0, y2=0, xy=0, w=0, wx=0, wy=0, x3=0, xy2=0, x2
 		surface <- "predict"
 	}
 	
-	C <- c(x, y, x2, y2, xy, w, wx, wy,x3, xy2, x2y, y3)
+	C <- c(x, y, x2, y2, xy, w, wx, wy, x3, xy2, x2y, y3)
 	
 	if (!model %in% c("absunc", "absdiff")) {
 		if (!is.null(fit) & model != "cubic" & model != "null") {
@@ -215,6 +233,59 @@ plotRSA <- function(x=0, y=0, x2=0, y2=0, xy=0, w=0, wx=0, wy=0, x3=0, xy2=0, x2
 		param <- FALSE
 		SP.text <- ""
 	}
+	
+	
+	# ---------------------------------------------------------------------
+	# Calculate positions of raw points
+
+	if (points$out.mark == TRUE & is.null(fit)) {
+		warning("Outliers can only be marked if an RSA-object is provided. Points are not plotted.")
+		points$show <- FALSE
+	}
+	if (points$show == TRUE | hull==TRUE) {
+		if (is.null(points$data)) stop("You must provide raw data if you want to plot raw points or the bagplot.")
+		if (points$out.mark == FALSE) {
+			data.used <- points$data
+		}
+		if (points$out.mark == TRUE & !is.null(fit)) {
+			data.used <- fit$data[, c(fit$IV1, fit$IV2, fit$DV)]	# this includes all data points
+		}
+	
+		if (points$jitter > 0) {
+			data.used[, 1] <- data.used[, 1] + rnorm(length(data.used[, 1]), 0, points$jitter)
+			data.used[, 2] <- data.used[, 2] + rnorm(length(data.used[, 2]), 0, points$jitter)
+		}
+	
+		if (points$value == "raw") {
+			zpoints <- as.vector(data.used[, 3])
+		} else if (points$value == "predicted") {
+			N <- colnames(data.used)
+			data.used2 <- add.variables(formula(paste0(N[3], " ~ ", N[1], "*", N[2])), data.used)
+			
+			# calculate predicted values
+			zpoints <- b0 + colSums(C*t(data.used2[, c(
+				N[1],
+				N[2],
+				paste0(N[1], "2"),
+				paste0(N[2], "2"),
+				paste0(N[1], "_", N[2]),
+				"W",
+				paste0("W_", N[1]),
+				paste0("W_", N[2]),
+				paste0(N[1], "3"),
+				paste0(N[1], "_", N[2], "2"),
+				paste0(N[1], "2", "_", N[2]),
+				paste0(N[2], "3")
+			)]))
+			
+			zpoints <- as.vector(zpoints)
+		}
+
+		xpoints <- as.vector(data.used[, 1])
+		ypoints <- as.vector(data.used[, 2])	
+	}
+	
+	
 		
 	# build data set
 	grid <- gridsize
@@ -272,54 +343,7 @@ plotRSA <- function(x=0, y=0, x2=0, y2=0, xy=0, w=0, wx=0, wy=0, x3=0, xy2=0, x2
 	if (length(pal) < 2) {legend <- FALSE}
 		
 	
-	# ---------------------------------------------------------------------
-	# Calculate positions of raw points
-
-	if (points$out.mark == TRUE & is.null(fit)) {
-		warning("Outliers can only be marked if an RSA-object is provided. Points are not plotted.")
-		points$show <- FALSE
-	}
-	if (points$show == TRUE) {
-		if (points$out.mark == FALSE) {
-			data.used <- points$data
-		}
-		if (points$out.mark == TRUE & !is.null(fit)) {
-			data.used <- fit$data.original
-		}
 	
-		if (points$jitter > 0) {
-			data.used[, 1] <- data.used[, 1] + rnorm(length(data.used[, 1]), 0, points$jitter)
-			data.used[, 2] <- data.used[, 2] + rnorm(length(data.used[, 2]), 0, points$jitter)
-		}
-	
-		if (points$value == "raw") {
-			zpoints <- as.vector(data.used[, 3])
-		} else if (points$value == "predicted") {
-			N <- colnames(data.used)
-			data.used2 <- add.variables(formula(paste0(N[3], " ~ ", N[1], "*", N[2])), data.used)
-			
-			# calculate predicted values
-			zpoints <- b0 + colSums(C*t(data.used2[, c(
-				N[1],
-				N[2],
-				paste0(N[1], "2"),
-				paste0(N[2], "2"),
-				paste0(N[1], "_", N[2]),
-				"W",
-				paste0("W_", N[1]),
-				paste0("W_", N[2]),
-				paste0(N[1], "3"),
-				paste0(N[1], "_", N[2], "2"),
-				paste0(N[1], "2", "_", N[2]),
-				paste0(N[2], "3")
-			)]))
-			
-			zpoints <- as.vector(zpoints)
-		}
-
-		xpoints <- as.vector(data.used[, 1])
-		ypoints <- as.vector(data.used[, 2])		
-	}
 	
 	# ---------------------------------------------------------------------
 	#  calculate bag plot: bag = outer, loop = inner
@@ -362,10 +386,10 @@ plotRSA <- function(x=0, y=0, x2=0, y2=0, xy=0, w=0, wx=0, wy=0, x3=0, xy2=0, x2
 		R <- range(DV2)
 		col2 <- as.character(cut(1:(R[2] - R[1] + 1), breaks=length(pal), labels=pal))
 		
-		open3d()
+		open3d(cex=cex)
 		rgl.viewpoint(-30, -90)
 		rgl.light(theta = 0, phi = 90, viewpoint.rel = TRUE, ambient = "#FF0000", diffuse = "#FFFFFF", specular = "#FFFFFF")
-		persp3d(P$x, P$y, DV2, xlab = xlab, ylab = ylab, zlab = zlab, color=col2[DV2 - R[1] + 1])
+		persp3d(P$x, P$y, DV2, xlab = xlab, ylab = ylab, zlab = zlab, color=col2[DV2 - R[1] + 1], main=main)
 
 		if (contour$show == TRUE) {
 		    contours <- contourLines(P, z=DV2)
@@ -378,13 +402,15 @@ plotRSA <- function(x=0, y=0, x2=0, y2=0, xy=0, w=0, wx=0, wy=0, x3=0, xy2=0, x2
 			if (points$out.mark == FALSE) {
 				points3d(data.frame(xpoints, ypoints, zpoints), col=points$color)
 			}
-			if (points$out.mark == TRUE & !is.null(fit)) {
-				colvec <- rep(points$color, nrow(fit$data.original))
-				colvec[fit$outliers] <- "red"
-				points3d(fit$data.original[, c(fit$IV1, fit$IV2, fit$DV)], col=colvec)
-				text3d(fit$data.original[fit$outliers, c(fit$IV1, fit$IV2, fit$DV)], col="red", texts="X")
-			} else {
-				warning("Please provide an RSA-object to mark outliers.")
+			if (points$out.mark == TRUE) {
+				if (!is.null(fit)) {
+					colvec <- rep(points$color, nrow(fit$data))
+					colvec[fit$outliers] <- "red"
+					points3d(fit$data[, c(fit$IV1, fit$IV2, fit$DV)], col=colvec)
+					text3d(fit$data[fit$outliers, c(fit$IV1, fit$IV2, fit$DV)], col="red", texts="X")
+				} else {
+					warning("Please provide an RSA-object to mark outliers.")
+				}
 			}
 		}
 		
@@ -400,10 +426,14 @@ plotRSA <- function(x=0, y=0, x2=0, y2=0, xy=0, w=0, wx=0, wy=0, x3=0, xy2=0, x2
 		
 	if (type == "3d") {
 				
-			mypanel2 <- function(x, y, z, xlim, ylim, zlim, xlim.scaled, ylim.scaled, zlim.scaled, axes, x.points=NULL, y.points=NULL, z.points=NULL, SPs="", ...) {
+			mypanel2 <- function(x, y, z, xlim, ylim, zlim, xlim.scaled, ylim.scaled, zlim.scaled, axes, axesList, x.points=NULL, y.points=NULL, z.points=NULL, SPs="", ...) {
 				
 				
 			   # rescale absolute x, y, and z values so that they fit into the box
+			   RESCALE.Z <- function(z1) {
+	              Z2 <- zlim.scaled[1] + diff(zlim.scaled) * (z1 - zlim[1]) / diff(zlim)
+				  return(Z2)
+			   }
 			   RESCALE <- function(n) {
 				  X2 <- xlim.scaled[1] + diff(xlim.scaled) * (n$X - xlim[1]) / diff(xlim)
 	              Y2 <- ylim.scaled[1] + diff(ylim.scaled) * (n$Y - ylim[1]) / diff(ylim)
@@ -416,14 +446,44 @@ plotRSA <- function(x=0, y=0, x2=0, y2=0, xy=0, w=0, wx=0, wy=0, x3=0, xy2=0, x2
 			
 			# ---------------------------------------------------------------------
 			# 1. Projection on bottom of cube
-			  if (project==TRUE) {
-				  for (a in axes) {
-					  #print(paste("Plotting projection of", names(a)))
-					  a0 <- RESCALE(getIntersect2(p0=a$p0, p1=a$p1, Z=zlim.final[1] + .01))
-		              panel.3dscatter(x = a0$X, y = a0$Y, z = a0$Z, xlim = xlim, ylim = ylim, zlim = zlim,
-		                              xlim.scaled = xlim.scaled, ylim.scaled = ylim.scaled, zlim.scaled = zlim.scaled, type="l", col.line=a$col, lty=a$lty, lwd=2, ...)
-				  } 
+			  if (length(project) > 0) {
+				  for (p in project) {
+					  if (p %in% c("LOC", "LOIC", "PA1", "PA2")) {
+						  a0 <- RESCALE(getIntersect2(p0=axesList[[p]]$p0, p1=axesList[[p]]$p1))
+						  if (nrow(a0) <= 1) break;
+							  panel.3dscatter(x = a0$X, y = a0$Y, z = rep(RESCALE.Z(min(zlim.final) + .01), nrow(a0)), 
+						  				xlim = xlim, ylim = ylim, zlim = zlim,
+			                            xlim.scaled = xlim.scaled, ylim.scaled = ylim.scaled, zlim.scaled = zlim.scaled, 
+										type="l", col.line=axesList[[p]]$col, lty=axesList[[p]]$lty, lwd=2, ...)
+					  }
+					  
+					  if (p == "hull") {
+					  	
+					  }
+				  }
 			  }
+				  
+				  
+				  # project contour on bottom
+				if (contour$show == TRUE | "contour" %in% project) {
+					# "abuse" ggplot to compute the contour lines
+					cs <- ggplot(new2, aes_string(x="x", y="y", fill="z", z="z")) + stat_contour(bins=ifelse(length(pal)>1, length(pal)+1, 8))
+					cLines <- ggplot_build(cs)
+					C0 <- cLines$data[[1]][, c("x", "y", "level", "group")]
+					colnames(C0) <- c("X", "Y", "Z", "group")	# C0 keeps the contour lines
+				
+				
+					if ("contour" %in% project) {	
+						for (cL in C0$group) {
+						  C1 <- RESCALE(C0[C0$group==cL, c("X", "Y", "Z")])
+						  panel.3dscatter(x = C1$X, y = C1$Y, z = rep(RESCALE.Z(min(zlim.final) + .01), nrow(C1)), 
+							 		xlim = xlim, ylim = ylim, zlim = zlim,
+				                    xlim.scaled = xlim.scaled, ylim.scaled = ylim.scaled, zlim.scaled = zlim.scaled,
+									type="l", col.line=contour$color, lty="solid", lwd=1, ...)
+						}	
+					}
+				}
+				  
 		  	
 				
 			   # ---------------------------------------------------------------------
@@ -451,17 +511,19 @@ plotRSA <- function(x=0, y=0, x2=0, y2=0, xy=0, w=0, wx=0, wy=0, x3=0, xy2=0, x2
 					  # 3. the surface
 						  panel.3dwire(x = x, y = y, z = z, xlim = xlim, ylim = ylim, zlim = zlim,
 		                           xlim.scaled = xlim.scaled, ylim.scaled = ylim.scaled, zlim.scaled = zlim.scaled,
-								   col=gridCol,  lwd=0.3, ...)
+								   col=gridCol, lwd=0.3, ...)
 								   
 								   
-					   
 					# ---------------------------------------------------------------------
 					# 4. plot of LOC and LOIC, and other axes
 						  for (a in axes) {
-							  a0 <- RESCALE(getIntersect2(p0=a$p0, p1=a$p1))
-				              panel.3dscatter(x = a0$X, y = a0$Y, z = a0$Z, xlim = xlim, ylim = ylim, zlim = zlim,
-				                              xlim.scaled = xlim.scaled, ylim.scaled = ylim.scaled, zlim.scaled = zlim.scaled,
-											  type="l", col.line=a$col, lty=a$lty, lwd=2, ...)
+							  if (!is.null(axesList[[a]])) {
+								  a0 <- RESCALE(getIntersect2(p0=axesList[[a]]$p0, p1=axesList[[a]]$p1))
+								  if (nrow(a0) <= 1) break;
+					              panel.3dscatter(x = a0$X, y = a0$Y, z = a0$Z, xlim = xlim, ylim = ylim, zlim = zlim,
+					                      	xlim.scaled = xlim.scaled, ylim.scaled = ylim.scaled, zlim.scaled = zlim.scaled, 
+											type="l", col.line=axesList[[a]]$col, lty=axesList[[a]]$lty, lwd=2, ...)
+							  }
 						  }   
 						  
   					# ---------------------------------------------------------------------
@@ -495,7 +557,7 @@ plotRSA <- function(x=0, y=0, x2=0, y2=0, xy=0, w=0, wx=0, wy=0, x3=0, xy2=0, x2
 					   }
 										  
 						  if (param == TRUE) {
-							  grid.text(SPs, .02, .95, just="left")
+							  grid.text(SPs, .02, .95, just="left", gp=gpar(cex=cex))
 						  }  
 						  
 						  
@@ -547,17 +609,19 @@ plotRSA <- function(x=0, y=0, x2=0, y2=0, xy=0, w=0, wx=0, wy=0, x3=0, xy2=0, x2
 					  		}	
 						
 					# ---------------------------------------------------------------------
-					# 7. plot contour lines:
-						if (contour$show == TRUE) {
-							cs <- ggplot(new2, aes_string(x="x", y="y", fill="z", z="z")) + stat_contour(bins=ifelse(length(pal)>1, length(pal)+1, 8))
-							cLines <- ggplot_build(cs)
-							C0 <- cLines$data[[1]][, c("x", "y", "level", "group")]
-							colnames(C0) <- c("X", "Y", "Z", "group")
+					# 7. plot contour lines on surface:
+					
+					if (contour$show == TRUE) {
+						# C0 keeps the contour lines and has been computed before
 							for (cL in C0$group) {
 							  C1 <- RESCALE(C0[C0$group==cL, c("X", "Y", "Z")])
-				             panel.3dscatter(x = C1$X, y = C1$Y, z = C1$Z, xlim = xlim, ylim = ylim, zlim = zlim,
-				                             xlim.scaled = xlim.scaled, ylim.scaled = ylim.scaled, zlim.scaled = zlim.scaled,
-											  type="l", col.line=contour$color, lty="solid", lwd=1, ...)
+							  
+							  if (contour$show == TRUE) {
+								  	panel.3dscatter(x = C1$X, y = C1$Y, z = C1$Z, 
+										xlim = xlim, ylim = ylim, zlim = zlim,
+				                        xlim.scaled = xlim.scaled, ylim.scaled = ylim.scaled, zlim.scaled = zlim.scaled,
+										type="l", col.line=contour$color, lty="solid", lwd=1, ...)
+								}								
 							}
 							
 							# highlight specific contour lines?
@@ -570,12 +634,10 @@ plotRSA <- function(x=0, y=0, x2=0, y2=0, xy=0, w=0, wx=0, wy=0, x3=0, xy2=0, x2
 												  type="l", col.line=contour$color, lty="solid", lwd=2, ...)
 								}
 								
-							}
-							
-							
+							}	
 						}
 						
-				}
+				}  # of mypanel2
 			
 
 
@@ -591,25 +653,11 @@ plotRSA <- function(x=0, y=0, x2=0, y2=0, xy=0, w=0, wx=0, wy=0, x3=0, xy2=0, x2
 				}
 				
 				axesList <- list()
-				if ("LOC" %in% axes) {axesList[["LOC"]]  <- list(p0=0, p1=1, lty="solid", col="grey")}
-				if ("LOIC" %in% axes) {axesList[["LOIC"]] <- list(p0=0, p1=-1, lty="dotted", col="grey")}
-				if ("PA1" %in% axes) {
-					if (x2 == y2) {
-						# print("Adjusting x2 to print PA1")
-						SP2 <- RSA.ST(x=x, y=y, xy=xy, x2=x2 + .001, y2=y2)
-						axesList[["PA1"]] <- list(p0=SP2$p10, p1=SP2$p11, lty="solid", col="black")
-						} else {
-							axesList[["PA1"]] <- list(p0=SP$p10, p1=SP$p11, lty="solid", col="black")
-						}					
-				}
-				if ("PA2" %in% axes) {
-					if (x2 == y2) {
-						# print("Adjusting x2 to print PA2")
-						SP2 <- RSA.ST(x=x, y=y, xy=xy, x2=x2 + .001, y2=y2)
-						axesList[["PA2"]] <- list(p0=SP2$p20, p1=SP2$p21, lty="dotted", col="black")
-					} else {
-						axesList[["PA2"]] <- list(p0=SP$p20, p1=SP$p21, lty="dotted", col="black")	
-					}
+				axesList[["LOC"]]  <- list(p0=0, p1=1, lty="solid", col="blue")
+				axesList[["LOIC"]] <- list(p0=0, p1=-1, lty="solid", col="blue")
+				if (x2 != y2) {
+					axesList[["PA1"]] <- list(p0=SP$p10, p1=SP$p11, lty="dotted", col="grey30")
+					axesList[["PA2"]] <- list(p0=SP$p20, p1=SP$p21, lty="dotted", col="grey30")	
 				}			
 				
 				
@@ -620,6 +668,11 @@ plotRSA <- function(x=0, y=0, x2=0, y2=0, xy=0, w=0, wx=0, wy=0, x3=0, xy2=0, x2
 					at <- seq(min(new2$z), max(new2$z), length.out=length(pal)-1)
 				}
 				
+				# define the appearance of the color legend
+				CK <- FALSE
+				if (legend == TRUE) {
+					CK <- list(labels=list(cex=cex))
+				}
 				
 			if (points$show == FALSE) {
 				p1 <- wireframe(z ~ x*y, new2,  drape=TRUE, 
@@ -627,33 +680,38 @@ plotRSA <- function(x=0, y=0, x2=0, y2=0, xy=0, w=0, wx=0, wy=0, x3=0, xy2=0, x2
 					xlab	= list(cex=cex, label=xlab, rot=label.rotation[["x"]]), 
 					ylab	= list(cex=cex, label=ylab, rot=label.rotation[["y"]]), 
 					zlab	= list(cex=cex, label=zlab, rot=label.rotation[["z"]]), zlim=zlim, 
+					main	= list(cex=cex.main, label=main),
 					screen	= rotation, 
-					at		= at, col.regions=pal, colorkey=legend, 
+					at		= at, col.regions=pal, colorkey=CK, 
 					par.settings = list(
 						axis.line = list(col = "transparent"), 
 						layout.heights = list(top.padding=pad, bottom.padding=pad), 
 						layout.widths=list(left.padding=pad, right.padding=pad)), 
-					axes	= axesList, 
+					axes	= axes,
+					axesList= axesList, 
 					SPs		= SP.text, 
-					panel.3d.wireframe = mypanel2, ...)
+					panel.3d.wireframe = mypanel2)
 								
 				#p1
 			} else {
+				
 				p1 <- wireframe(z ~ x*y, new2,  drape=TRUE, 
 					scales 	= list(arrows = FALSE, cex=cex, col = "black", font = 1, tck=tck, distance=distance), 
 					xlab	= list(cex=cex, label=xlab, rot=label.rotation[["x"]]), 
 					ylab	= list(cex=cex, label=ylab, rot=label.rotation[["y"]]), 
 					zlab	= list(cex=cex, label=zlab, rot=label.rotation[["z"]]), zlim=zlim, 
+					main	= list(cex=cex.main, label=main),
 					screen	= rotation,
-					at		= at, col.regions=pal, colorkey=legend, 
+					at		= at, col.regions=pal, colorkey=CK, 
 					par.settings = list(
 						axis.line = list(col = "transparent"), 
 						layout.heights = list(top.padding=pad, bottom.padding=pad), 
 						layout.widths=list(left.padding=pad, right.padding=pad)), 
-					axes	= axesList, 
+					axes	= axes,
+					axesList= axesList, 
 					SPs		= SP.text, 
 					panel.3d.wireframe = mypanel2,
-					x.points=xpoints, y.points=ypoints, z.points=zpoints, ...)
+					x.points=xpoints, y.points=ypoints, z.points=zpoints)
 			}
 				
 	}  # of type == "3d"
@@ -706,7 +764,7 @@ plotRSA <- function(x=0, y=0, x2=0, y2=0, xy=0, w=0, wx=0, wy=0, x3=0, xy2=0, x2
 				p1 <- p1+ geom_abline(data=data.frame(SP[c("p20", "p21")]), aes_string(intercept="p20", slope="p21"), linetype="dotted")
 			}
 			
-			if (param==TRUE & !any(is.na(SP[c("X0", "Y0")])) & !model %in% c("RR", "sqdiff", "SSD", "SRSD", "SRR", "SRRR")) {
+			if (param==TRUE & !any(is.na(SP[c("X0", "Y0")])) & !model %in% c("RR", "SQD", "SSQD", "SRSQD", "SRR", "SRRR")) {
 				p1 <- p1 + annotate("point", x=SP$X0, y=SP$Y0, z=max(new2$z))
 			}
 				
@@ -716,11 +774,11 @@ plotRSA <- function(x=0, y=0, x2=0, y2=0, xy=0, w=0, wx=0, wy=0, x3=0, xy2=0, x2
 					p1 <- p1 + annotate("point", x=xpoints, y=ypoints, color=points$color, size=3*points$cex)
 				}
 				if (points$out.mark==TRUE) {
-					colvec <- rep(points$color, nrow(fit$data.original))
+					colvec <- rep(points$color, nrow(fit$data))
 					colvec[fit$outliers] <- "red"
-					shapevec <- rep(19, nrow(fit$data.original))
+					shapevec <- rep(19, nrow(fit$data))
 					shapevec[fit$outliers] <- 4
-					p1 <- p1 + annotate("point", x=fit$data.original[, fit$IV1], y=fit$data.original[, fit$IV2], color=colvec, size=3*points$cex, shape=shapevec)
+					p1 <- p1 + annotate("point", x=fit$data[, fit$IV1], y=fit$data[, fit$IV2], color=colvec, size=3*points$cex, shape=shapevec)
 				}
 			}
 			
@@ -736,6 +794,9 @@ plotRSA <- function(x=0, y=0, x2=0, y2=0, xy=0, w=0, wx=0, wy=0, x3=0, xy2=0, x2
 				p1 <- p1 + annotate("errorbarh", x=SP$X0, y=SP$Y0, xmin=PAR[PAR$label=="X0", "ci.lower"], xmax=PAR[PAR$label=="X0", "ci.upper"], z=max(new2$z), height=.3)
 			}
 			
+			# Title
+			if (main != "") p1 <- p1 + ggtitle(main)
+			
 		}
 	}
 	
@@ -744,22 +805,23 @@ plotRSA <- function(x=0, y=0, x2=0, y2=0, xy=0, w=0, wx=0, wy=0, x3=0, xy2=0, x2
 }
 
 
-#' @S3method plot RSA
+#' @export
 
 # Purpose: Extract the model parameters, xlim, xlab, etc. from the fitted object and give it to the plotRSA function
 plot.RSA <- function(x, ...) {
 	fit <- x
 	
 	extras <- match.call(expand.dots = FALSE)$...
-	if (is.null(extras)) extras <- list()
-	if (is.null(extras$model)) extras$model <- "full"
+
+	if (is.null(extras)) {extras <- list()}
+	if (is.null(extras[["model"]])) {extras[["model"]] <- "full"}
 	
 	C <- coef(fit$models[[extras$model]])
 	if (fit$models[[extras$model]]@Options$estimator != "DWLS") {
-		extras$b0 <- as.numeric(ifelse(is.na(C[paste0(fit$DV, "~1")]), 0, C[paste0(fit$DV, "~1")]))
+		extras[["b0"]] <- as.numeric(ifelse(is.na(C[paste0(fit$DV, "~1")]), 0, C[paste0(fit$DV, "~1")]))
 	} else {			
 		# the threshold is the negative of the intercept ...
-		extras$b0 <- -as.numeric(ifelse(is.na(C[paste0(fit$DV, "|t1")]), 0, C[paste0(fit$DV, "|t1")]))
+		extras[["b0"]] <- -as.numeric(ifelse(is.na(C[paste0(fit$DV, "|t1")]), 0, C[paste0(fit$DV, "|t1")]))
 	}
 	extras$x <- as.numeric(ifelse(is.na(C["b1"]), 0, C["b1"]))
 	extras$y <- as.numeric(ifelse(is.na(C["b2"]), 0, C["b2"]))
@@ -775,10 +837,30 @@ plot.RSA <- function(x, ...) {
 	extras$xy2 <- as.numeric(ifelse(is.na(C["b10"]), 0, C["b10"]))
 	extras$x2y <- as.numeric(ifelse(is.na(C["b11"]), 0, C["b11"]))
 	extras$y3 <- as.numeric(ifelse(is.na(C["b12"]), 0, C["b12"]))
+	
+	if (is.null(extras[["xlab"]])) {extras[["xlab"]] <- fit$IV1}
+	if (is.null(extras[["ylab"]])) {extras[["ylab"]] <- fit$IV2}
+	if (is.null(extras[["zlab"]])) {extras[["zlab"]] <- fit$DV}
+		
+	extras$fit <- fit
+
+	if (is.null(extras$points)) {
+		extras$points <- list(show=TRUE, value="raw", jitter=0, color="black", cex=.5, out.mark=FALSE)
+	}
+	if (is.null(extras$points$out.mark)) extras$points$out.mark <- FALSE
+
+	if (extras$points$out.mark == FALSE) {
+		data.used <- fit$data[fit$data$out==FALSE, ]
+	}
+	if (extras$points$out.mark == TRUE) {
+		data.used <- fit$data
+	}
+	
+	extras$points$data <- data.used[, c(fit$IV1, fit$IV2, fit$DV, colnames(fit$data)[which(!colnames(fit$data) %in% c(fit$IV1, fit$IV2, fit$DV))])]
 
 	adjust <- FALSE
 	if (is.null(extras$xlim)) {
-		extras$xlim <- c(min(fit$data[, fit$IV1], na.rm=TRUE), max(fit$data[, fit$IV1], na.rm=TRUE))
+		extras$xlim <- c(min(data.used[, fit$IV1], na.rm=TRUE), max(data.used[, fit$IV1], na.rm=TRUE))
 		# expand range by 20% at each end
 		extras$xlim[1] <- extras$xlim[1]*ifelse(extras$xlim[1]<0, 1.1, 0.9)
 		extras$xlim[2] <- extras$xlim[2]*ifelse(extras$xlim[2]<0, 0.9, 1.1)
@@ -786,7 +868,7 @@ plot.RSA <- function(x, ...) {
 	}
 	
 	if (is.null(extras$ylim)) {
-		extras$ylim <- c(min(fit$data[, fit$IV2], na.rm=TRUE), max(fit$data[, fit$IV2], na.rm=TRUE))
+		extras$ylim <- c(min(data.used[, fit$IV2], na.rm=TRUE), max(data.used[, fit$IV2], na.rm=TRUE))
 		extras$ylim[1] <- extras$ylim[1]*ifelse(extras$ylim[1]<0, 1.1, 0.9)
 		extras$ylim[2] <- extras$ylim[2]*ifelse(extras$ylim[2]<0, 0.9, 1.1)
 		adjust <- TRUE
@@ -797,47 +879,6 @@ plot.RSA <- function(x, ...) {
 		extras$xlim[2] <- extras$ylim[2] <- max(extras$xlim[2], extras$ylim[2])
 	}
 	
-	if (is.null(extras$xlab)) extras$xlab <- fit$IV1
-	if (is.null(extras$ylab)) extras$ylab <- fit$IV2
-	if (is.null(extras$zlab)) extras$zlab <- fit$DV
-		
-	if (is.null(extras$points)) {
-		extras$points <- list(show=TRUE, value="raw", jitter=0, color="black", cex=.5, out.mark=FALSE)
-	}
-	extras$points$data <- fit$data[, c(fit$IV1, fit$IV2, fit$DV, colnames(fit$data)[which(!colnames(fit$data) %in% c(fit$IV1, fit$IV2, fit$DV))])]
-	
-	extras$fit <- fit
-	
-	do.call(plotRSA, as.list(extras))
+	do.call(plotRSA, as.list(extras), envir = parent.frame())
 }
 
-
-#plot(r1, type="3d", points=list(show=TRUE, cex=2))
-
-#plotRSA(x=.05, x2=.1, xy=.20, rotation=list(x=-50, y=58, z=36), legend=FALSE, points=list(data=data.frame(x=-1:3, y=-1:3, z=0), show=TRUE))
-
-#plot(r1)
-#plot(r1, points=list(show=TRUE))
-#plotRSA(x=.05, x2=.1, xy=.20, type="3", gridsize=21)
-
-#demoRSA(x=.625, y=.519, x2=-.196, xy=.285, y2=-.167)
-#demoRSA(x=.625, y=.519, x2=-.196, xy=.285, y2=-.167, type="c")
-#
-#plotRSA(fit=r1, type="3d", points=list(show=TRUE), rotation=list(x=-58, y=50, z=26))
-#plotRSA(x=.05, x2=.1, xy=.20, rotation=list(x=-50, y=58, z=36), legend=FALSE, type="c")
-#plotRSA(x=.05, x2=.1, xy=.20, rotation=list(x=-50, y=58, z=36), legend=FALSE, type="3d")
-#
-## exampe of Edwards, 2002, Figure 3
-#p1 <- plotRSA(b0=5.628, x=.314, y=-.118, x2=-.145, y2=-.102, xy=.299, legend=FALSE, type="3d")
-#plotRSA(b0=5.628, x=.314, y=-.118, x2=-.145, y2=-.102, xy=.299, legend=FALSE, type="c")
-
-#RSA.ST(x=.314, y=-.118, x2=-.145, xy=.299, y2=-.102)
-
-
-## Minimal example
-#x <- y <- z <- c(1,2,3)
-#df <- data.frame(x, y, z)
-#wireframe(z ~ x*y, df,  scales = list(arrows = FALSE, col = "black", font = 1, tck = c(1, 1, 1), distance =c(1, 1, 1)))
-#
-#
-#xyplot(z ~ x, df,  scales = list(arrows = FALSE, cex=cex, col = "black", font = 1, tck=tck), par.settings=list(axis.components=list(left=list(pad1=2))))
\ No newline at end of file
